@@ -1,5 +1,5 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2004 Jiri (George) Lebl
+ * Copyright (C) 1997-2005 Jiri (George) Lebl
  *
  * Author: Jiri (George) Lebl
  *
@@ -33,18 +33,7 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <stdio.h>
-
-#ifdef USE_NCURSES
-#ifdef INC_NCURSES
-#include <ncurses/curses.h>
-#include <ncurses/term.h>
-#else
-#include <curses.h>
-#include <term.h>
-#endif
-#else
-#include <termcap.h>
-#endif
+#include <locale.h>
 
 #include "calc.h"
 #include "eval.h"
@@ -61,11 +50,23 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 
+#ifdef USE_NCURSES
+#ifdef INC_NCURSES
+#include <ncurses/curses.h>
+#include <ncurses/term.h>
+#else
+#include <curses.h>
+#include <term.h>
+#endif
+#else
+#include <termcap.h>
+#endif
+
 /*Globals:*/
 
 /*calculator state*/
 calcstate_t curstate={
-	256,
+	128,
 	12,
 	FALSE,
 	FALSE,
@@ -116,6 +117,13 @@ gel_printout_infos (void)
 	errors_printed = 0;
 }
 
+void
+gel_call_help (const char *function)
+{
+	/* No extra processing of help */
+}
+
+
 static int
 long_get_term_width (void)
 {
@@ -175,15 +183,9 @@ nop (void)
 static const char *
 get_version_details (void)
 {
-#ifndef HAVE_MPFR
-	return _("\nNote: Compiled without MPFR (some operations may be slow) "
-		 "see www.mpfr.org");
-#else
 	return "";
-#endif
 }
 
-
 int
 main(int argc, char *argv[])
 {
@@ -194,7 +196,9 @@ main(int argc, char *argv[])
 	char *file;
 	FILE *fp;
 	gboolean do_compile = FALSE;
+	gboolean do_gettext = FALSE;
 	gboolean be_quiet = FALSE;
+	char *exec = NULL;
 
 	genius_is_gui = FALSE;
 
@@ -215,11 +219,39 @@ main(int argc, char *argv[])
 			files = g_slist_append(files,argv[i]);
 		else if(strcmp(argv[i],"--")==0)
 			lastarg = TRUE;
-		else if(sscanf(argv[i],"--precision=%d",&val)==1)
+		else if(sscanf(argv[i],"--precision=%d",&val)==1) {
+			if (val < 60 || val > 16384) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--precision", 60, 16384, 128);
+				val = 128;
+			}
 			curstate.float_prec = val;
-		else if(sscanf(argv[i],"--maxdigits=%d",&val)==1)
+		} else if (strcmp (argv[i], "--precision")==0 && i+1 < argc) {
+			val = 0;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 60 || val > 16384) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--precision", 60, 16384, 128);
+				val = 128;
+			}
+			curstate.float_prec = val;
+		} else if(sscanf(argv[i],"--maxdigits=%d",&val)==1) {
+			if (val < 0 || val > 256) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--maxdigits", 0, 256, 12);
+				val = 12;
+			}
 			curstate.max_digits = val;
-		else if(strcmp(argv[i],"--floatresult")==0)
+		} else if (strcmp (argv[i], "--maxdigits")==0 && i+1 < argc) {
+			val = -1;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 0 || val > 256) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--maxdigits", 0, 256, 12);
+				val = 12;
+			}
+			curstate.max_digits = val;
+		} else if(strcmp(argv[i],"--floatresult")==0)
 			curstate.results_as_floats = TRUE;
 		else if(strcmp(argv[i],"--nofloatresult")==0)
 			curstate.results_as_floats = FALSE;
@@ -231,15 +263,33 @@ main(int argc, char *argv[])
 			curstate.full_expressions = TRUE;
 		else if(strcmp(argv[i],"--nofullexp")==0)
 			curstate.full_expressions = FALSE;
-		else if(sscanf(argv[i],"--maxerrors=%d",&val)==1)
+		else if(sscanf(argv[i],"--maxerrors=%d",&val)==1) {
+			if (val < 0) {
+				g_printerr (_("%s should be greater then or equal to %d, using %d"),
+					    "--maxerrors", 0, 5);
+				val = 5;
+			}
 			curstate.max_errors = val;
-		else if(strcmp(argv[i],"--mixed")==0)
+		} else if (strcmp (argv[i], "--maxerrors")==0 && i+1 < argc) {
+			val = -1;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 0) {
+				g_printerr (_("%s should be greater then or equal to %d, using %d"),
+					    "--maxerrors", 0, 5);
+				val = 5;
+			}
+			curstate.max_errors = val;
+		} else if(strcmp(argv[i],"--mixed")==0)
 			curstate.mixed_fractions = TRUE;
 		else if(strcmp(argv[i],"--nomixed")==0)
 			curstate.mixed_fractions = FALSE;
-		else if(sscanf(argv[i],"--intoutbase=%d",&val)==1)
+		else if(sscanf(argv[i],"--intoutbase=%d",&val)==1) {
 			curstate.integer_output_base = val;
-		else if(strcmp(argv[i],"--readline")==0)
+		} else if (strcmp (argv[i], "--intoutbase")==0 && i+1 < argc) {
+			val = 10;
+			sscanf (argv[++i],"%d",&val);
+			curstate.integer_output_base = val;
+		} else if(strcmp(argv[i],"--readline")==0)
 			use_readline = TRUE;
 		else if(strcmp(argv[i],"--noreadline")==0)
 			use_readline = FALSE;
@@ -247,18 +297,35 @@ main(int argc, char *argv[])
 			do_compile = TRUE;
 		else if(strcmp(argv[i],"--nocompile")==0)
 			do_compile = FALSE;
+		else if(strcmp(argv[i],"--gettext")==0)
+			do_gettext = TRUE;
+		else if(strcmp(argv[i],"--nogettext")==0)
+			do_gettext = FALSE;
 		else if(strcmp(argv[i],"--quiet")==0)
 			be_quiet = TRUE;
 		else if(strcmp(argv[i],"--noquiet")==0)
 			be_quiet = FALSE;
-		else {
-			if(strcmp(argv[i],"--help")!=0) {
+		else if (strncmp (argv[i], "--exec=", strlen ("--exec=")) == 0) {
+			exec = g_strdup ((argv[i])+strlen("--exec="));
+		} else if (strcmp (argv[i], "--exec") && i+1 < argc) {
+			exec = g_strdup (argv[++i]);
+		} else if (strcmp (argv[i], "--version") == 0) {
+			g_print (_("Genius %s\n"
+				   "%s%s\n"),
+				 VERSION,
+				 COPYRIGHT_STRING,
+				 get_version_details ());
+			exit (0);
+		} else {
+			if (strcmp (argv[i], "--help") != 0) {
 				g_printerr ("Unknown argument '%s'!\n\n",
 					argv[i]);
 			}
 			g_print (_("Genius %s usage:\n\n"
 				   "genius [options] [files]\n\n"
-				   "\t--precision=num   \tFloating point precision [256]\n"
+				   "\t--help            \tPrint this help\n"
+				   "\t--version         \tPrint version number\n"
+				   "\t--precision=num   \tFloating point precision [128]\n"
 				   "\t--maxdigits=num   \tMaximum digits to display (0=no limit) [0]\n"
 				   "\t--[no]floatresult \tAll results as floats [OFF]\n"
 				   "\t--[no]scinot      \tResults in scientific notation [OFF]\n"
@@ -268,13 +335,24 @@ main(int argc, char *argv[])
 				   "\t--intoutbase=num  \tBase to use to print out integers [10]\n"
 				   "\t--[no]readline    \tUse readline if it is available [ON]\n"
 				   "\t--[no]compile     \tCompile everything and dump it to stdout [OFF]\n"
+				   "\t--[no]gettext     \tDump help/error strings in fake .c file to\n"
+				   "\t                  \tstdout (for use with gettext) [OFF]\n"
 				   "\t--[no]quiet       \tBe quiet during non-interactive mode,\n"
-				   "\t                  \t(always on when compiling) [OFF]\n\n"),
+				   "\t                  \t(always on when compiling) [OFF]\n"
+				   "\t--exec=expr       \tExecute an expression\n\n"),
 				 VERSION);
-			exit(1);
+			if (strcmp (argv[i], "--help") != 0)
+				exit (1);
+			else
+				exit (0);
 		}
 	}
 
+	if (files != NULL && exec != NULL) {
+		g_printerr (_("Can't specify both an expression and files to execute on the command line"));
+		exit (1);
+	}
+
 #if 0
 	{
 		/* FIXME: use this for option parsing,
@@ -292,9 +370,9 @@ main(int argc, char *argv[])
 
 	gel_read_plugin_list();
 
-	if (do_compile)
+	if (do_compile || do_gettext)
 		be_quiet = TRUE;
-	inter = isatty(0) && !files && !do_compile;
+	inter = isatty(0) && !files && !exec && !(do_compile || do_gettext);
 	/*interactive mode, print welcome message*/
 	if (inter) {
 		g_print (_("Genius %s\n"
@@ -326,7 +404,7 @@ main(int argc, char *argv[])
 	  dictionary*/
 	d_singlecontext ();
 
-	if ( ! do_compile) {
+	if ( ! (do_compile || do_gettext)) {
 		/*
 		 * Read main library
 		 */
@@ -377,15 +455,19 @@ main(int argc, char *argv[])
 			}
 		} while(!fp && files);
 		if(!fp && !files) {
-			if ( ! do_compile)
+			if ( ! (do_compile || do_gettext))
 				gel_save_plugins ();
 			return 0;
 		}
+	} else if (exec != NULL) {
+		fp = NULL;
+		gel_push_file_info("expr",1);
 	} else {
 		fp = stdin;
 		gel_push_file_info(NULL,1);
 	}
-	gel_lexer_open(fp);
+	if (fp != NULL)
+		gel_lexer_open(fp);
 	if(inter && use_readline) {
 		init_inter();
 	}
@@ -394,6 +476,13 @@ main(int argc, char *argv[])
 	
 	rl_event_hook = nop;
 
+	if (exec != NULL) {
+		line_len_cache = -1;
+		gel_evalexp (exec, NULL, main_out, NULL, FALSE, NULL);
+		line_len_cache = -1;
+		goto after_exec;
+	}
+
 	for(;;) {
 		for(;;) {
 			if(inter && use_readline) /*use readline mode*/ {
@@ -437,31 +526,23 @@ main(int argc, char *argv[])
 				}
 			} while(!fp && files);
 			if(!fp && !files) {
-				if (do_compile) {
-					gel_push_file_info(NULL,0);
-					gel_compile_all_user_funcs(stdout);
-					gel_pop_file_info();
-					/* if we have gotten errors then
-					   signal by returning a 1 */
-					if(total_errors_printed)
-						return 1;
-				} else {
-					gel_save_plugins ();
-				}
-				return 0;
+				goto after_exec;
 			}
 			gel_lexer_open(fp);
 		} else
 			break;
 	}
 
+after_exec:
+
 	gel_printout_infos ();
 	
-	gel_lexer_close(fp);
+	if (fp != NULL)
+		gel_lexer_close(fp);
 	/*if(fp != stdin)
 		fclose(fp);*/
 	
-	if(do_compile) {
+	if (do_compile) {
 		gel_push_file_info(NULL,0);
 		gel_compile_all_user_funcs(stdout);
 		gel_pop_file_info();
@@ -469,6 +550,15 @@ main(int argc, char *argv[])
 		   signal by returning a 1 */
 		if(total_errors_printed)
 			return 1;
+	} else if (do_gettext) {
+		gel_push_file_info (NULL, 0);
+		gel_dump_strings_from_help (stdout);
+		gel_dump_strings_from_user_funcs (stdout);
+		gel_pop_file_info ();
+		/* if we have gotten errors then
+		   signal by returning a 1 */
+		if (total_errors_printed)
+			return 1;
 	} else {
 		gel_save_plugins ();
 	}
